import boto3


class VPCAccessor(object):
    def __init__(self, region):
        self.region = region
    
    def get_ec2_resources(self):
        return boto3.resource("ec2", self.region)
    
    def get_vpcs(self):
        ec2_resources = self.get_ec2_resources()
        vpcs = {}
        for vpc in ec2_resources.vpcs.all():
            vpcs[vpc.id] = None
            if vpc.is_default:
                vpcs[vpc.id] = "Default"
            if vpc.tags:
                name_tags = [tag["Value"] for tag in vpc.tags if tag["Key"] == "Name"]
                if name_tags:
                    vpcs[vpc.id] = name_tags[0]
        return vpcs
    
    def get_subnets(self, vpc_id, public=True, instance_type=None):
        ec2_resources = self.get_ec2_resources()
        vpc = ec2_resources.Vpc(vpc_id)
        if instance_type:
            supported_subnets = self.get_subnets_for_instance_type(instance_type)
        subnets = {}
        for subnet in vpc.subnets.all():
            if instance_type and subnet.availability_zone not in supported_subnets:
                continue
            subnets[subnet.id] = subnet.availability_zone
        public_subnets = {}
        private_subnets = {}
        
        route_tables = ec2_resources.route_tables.all()
        main_route_table_public = False
        for route_table in route_tables:
            if route_table.vpc.id != vpc_id:
                continue
            is_public = self.is_route_table_public(route_table)
            for association in route_table.associations:
                if association.main:
                    main_route_table_public = is_public
                if association.subnet:
                    if is_public:
                        public_subnets[association.subnet.id] = subnets.pop(association.subnet.id, None)
                    else:
                        private_subnets[association.subnet.id] = subnets.pop(association.subnet.id, None)
        if main_route_table_public:
            public_subnets.update(subnets)
        else:
            private_subnets.update(subnets)
    
        if public:
            return public_subnets
        else:
            return private_subnets
    
    def is_route_table_public(self, route_table):
        return any(ra.get('DestinationCidrBlock') == '0.0.0.0/0' and ra.get('GatewayId') is not None for ra in route_table.routes_attribute)
    
    def get_subnets_for_instance_type(self, instance_type):
        client = boto3.client("ec2", self.region)
        ret = client.describe_instance_type_offerings(LocationType="availability-zone", Filters=[{"Name": "instance-type", "Values": [instance_type]}])
        return [i["Location"] for i in ret["InstanceTypeOfferings"]]



import time
import boto3

from datetime import timedelta, datetime
from botocore.exceptions import ParamValidationError
from choicesenum import ChoicesEnum
from eb_create_environment.utils import generate_secure_password
from eb_create_environment.vpc import VPCAccessor


BASE_PARAMS = [
    'AllocatedStorage',
    'DBInstanceClass',
    'MasterUsername',
    'BackupRetentionPeriod',
    'MultiAZ',
    'AutoMinorVersionUpgrade',
    'PubliclyAccessible',
    'StorageType',
    'StorageEncrypted',
    'CopyTagsToSnapshot',
    'MonitoringInterval',
    'DeletionProtection',
    'MaxAllocatedStorage',
]

# Currently not in use
EXTENDED_PARAMS = dict(
    AvailabilityZone='string',
    PreferredMaintenanceWindow='string',
    PreferredBackupWindow='string',
    Iops=0,
    DBClusterIdentifier='string',
    Tags=[
        {
            'Key': 'string',
            'Value': 'string'
        },
    ],
    EnableCustomerOwnedIp=False,
    OptionGroupName='string',
    CharacterSetName='string',
    TdeCredentialArn='string',
    TdeCredentialPassword='string',
    KmsKeyId='string',
    ProcessorFeatures=[
        {
            'Name': 'string',
            'Value': 'string'
        },
    ],
    EnableCloudwatchLogsExports=[
        'string',
    ],
    PerformanceInsightsKMSKeyId='string',
    PerformanceInsightsRetentionPeriod=7,
    Domain='string',
    MonitoringRoleArn='string',
    DomainIAMRoleName='string',
    PromotionTier=123,  # aurora
    Timezone='string',  # sqlserver
    EnableIAMDatabaseAuthentication=True | False,
    EnablePerformanceInsights=True | False,
)

POSTGRES_PARAMS = [
    'DBName',
    'Engine',
    'EngineVersion',
    'Port',
    'DBParameterGroupName',
    'LicenseModel',
]

ORACLE_PARAMS = [
    'DBName',
    'Engine',
    'EngineVersion',
    'Port',
    'DBParameterGroupName',
    'LicenseModel',
    'NcharCharacterSetName',
]


class Engine(ChoicesEnum):
    postgres = "postgres"
    oracle = "oracle"


PARAMS_BY_ENGINE = {
    Engine.postgres: POSTGRES_PARAMS,
}

ENGINE_NAME_LOOKUP = {
    Engine.postgres: "Postgres",
}


class DatabaseInitializer(object):
    def __init__(self, region, config, engine, vpc_id, environment_name, application_security_group_id):
        self.region = region

        self.password = generate_secure_password()
        self.engine = engine
        self.config = config

        self.client = boto3.client("rds", self.region)
        self.vpc_id = vpc_id
        self.vpc_subnet = ""  # TODO: this
        self.environment_name = environment_name
        self.db_name = f"{self.environment_name}-db"
        self.application_security_group_id = application_security_group_id

    def create_db_security_group(self):
        ec2_client = boto3.client('ec2', self.region)
        security_group_name = f"{self.environment_name}-db"
        response = ec2_client.create_security_group(
            GroupName=security_group_name,
            Description=f"Database security group for {self.environment_name}",
            VpcId=self.vpc_id,
            TagSpecifications=[{
                "ResourceType": "security-group",
                "Tags": [{"Key": "Name", "Value": security_group_name}]
            }]
        )
        security_group_id = response['GroupId']
        port = self.get_config_params()["Port"]
        ec2_client.authorize_security_group_ingress(
            GroupId=security_group_id,
            IpPermissions=[
                {'IpProtocol': 'tcp',
                 'FromPort': port,
                 'ToPort': port,
                 'UserIdGroupPairs': [{'GroupId': self.application_security_group_id}]},
            ]
        )
        return security_group_id

    def create_db(self):
        vpc_security_groups = [self.create_db_security_group()]
        db_subnet_group = self.get_db_subnet_group()
        if not db_subnet_group:
            db_subnet_group = self.create_db_subnet_group()
        try:
            params = dict(
                DBInstanceIdentifier=self.db_name,
                MasterUserPassword=self.password,
                # DBSecurityGroups=db_security_groups,
                VpcSecurityGroupIds=vpc_security_groups,
                DBSubnetGroupName=db_subnet_group,
                **self.get_config_params(),
            )
            self.client.create_db_instance(**params)
        except ParamValidationError:
            print(self.get_config_params())
            raise
        host = self.get_host_from_response()
        print("Waiting for Database")
        return self.get_db_url(params['MasterUsername'], host, params['Port'])

    def get_config_params(self):
        config_engine_name = ENGINE_NAME_LOOKUP[self.engine]
        base_params = {
            param: self.config['RDS'][param] for param in BASE_PARAMS
        }
        engine_params = {
            param: self.config['RDS'][config_engine_name][param] for param in PARAMS_BY_ENGINE[self.engine]
        }
        return {
            **base_params,
            **engine_params,
        }

    def get_db_url(self, user, host, port):
        postgres_db_name = self.config["RDS"]["Postgres"]["DBName"]
        if self.engine == Engine.postgres:
            return f"postgres://{user}:{self.password}@{host}:{port}/{postgres_db_name}?sslmode=require"
        else:
            return ""

    def get_host_from_response(self):
        timeout = timedelta(minutes=15)
        start = datetime.now()
        host = ""
        while True:
            if datetime.now() > start + timeout:
                break
            response = self.client.describe_db_instances()
            for db in response['DBInstances']:
                if db['DBInstanceIdentifier'] == self.db_name:
                    try:
                        host = db['Endpoint']['Address'] or ""
                    except:
                        pass
            if host:
                break
            time.sleep(10)
        return host

    def get_db_subnet_group(self):
        response = self.client.describe_db_subnet_groups()
        if 'DBSubnetGroups' in response:
            vpc_subnet_groups = [
                subnet_group['DBSubnetGroupName'] for subnet_group in response['DBSubnetGroups']
                if subnet_group['VpcId'] == self.vpc_id
            ]
            if vpc_subnet_groups:
                return vpc_subnet_groups[0]
        return None
    
    def create_db_subnet_group(self):
        vpc = VPCAccessor(self.region)
        subnet_ids = list(vpc.get_subnets(self.vpc_id))
        subnet_group_name = f"default-{self.vpc_id}"
        self.client.create_db_subnet_group(
            DBSubnetGroupName=subnet_group_name,
            DBSubnetGroupDescription=f"All subnets for {self.vpc_id}",
            SubnetIds=subnet_ids,
        )
        return subnet_group_name



import random
import string


def generate_secure_password(length=32):
    return "".join(random.choices(string.ascii_letters + string.digits, k=length))


def poll_options(options):
    while True:
        for index, option in enumerate(options):
            print(f"[{index+1}]: {option}")
        val = input("Selection: ")
        try:
            selected_option = options[int(val)+1]
            break
        except TypeError:
            pass
        except ValueError:
            pass
        except IndexError:
            pass
    return selected_option



import os
import setuptools


with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

setuptools.setup(
    author="Zagaran, Inc.",
    author_email="info@zagaran.com",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    description="Tool to create an Elastic Beanstalk environment and linked database using sensible defaults",
    entry_points={
        "console_scripts": ["eb-create-environment=eb_create_environment.script:main"],
    },
    install_requires=["boto3", "choicesenum", "ipython", "jedi", "pyyaml"],
    keywords="aws eb elastic beanstalk rds database create environment",
    license="MIT",
    long_description=README,
    long_description_content_type="text/markdown",
    name="eb-create-environment",
    packages=setuptools.find_packages(),
    package_data={'': ['default_config.yml']},
    include_package_data=True,
    python_requires='>=3.6',
    url="https://github.com/zagaran/eb-environment-creation",
    version="0.0.4",
)



import argparse
import boto3
import os
import sys
import yaml

from eb_create_environment.database import DatabaseInitializer, Engine
from eb_create_environment.eb_setup import EBInitializer
from eb_create_environment.vpc import VPCAccessor


DEFAULT_CONFIG_FILE_PATH = "default_config.yml"
EB_GLOBAL_CONFIG_DIRECTORY = ".elasticbeanstalk"
EB_GLOBAL_CONFIG_FILE_PATH = os.path.join(EB_GLOBAL_CONFIG_DIRECTORY, "config.yml")


class SetupWrapper(object):
    def __init__(self):
        parser = argparse.ArgumentParser(description="Set up linked EB and RDS instances")
        parser.add_argument(
            "-c", "--config",
            default=None,
            help="Specify a custom config file",
        )
        parser.add_argument(
            "-a", "--application_name",
            default=None,
            help="Elastic Beanstalk application name",
        )
        parser.add_argument(
            "-e", "--environment_name",
            default=None,
            help="Elastic Beanstalk environment name",
        )
        parser.add_argument(
            "-p", "--profile",
            default=None,
            help="Specify an AWS profile from your credential file",
        )
        parser.add_argument(
            "-r", "--region",
            default=None,
            help="Specify an AWS region region",
        )
        parser.add_argument(
            "--db-only",
            default=False,
            action="store_true",
            help="Skip setup of application and environment. Requires application and environment to exist already."
        )
        parser.add_argument(
            "--no-db",
            default=False,
            action="store_true",
            help="Skip setup of the database.  Cannot be used with `--db-only`"
        )
        parser.add_argument(
            "--print-default-config",
            default=False,
            action="store_true",
            help="Print default config and exit"
        )
        args = parser.parse_args()
        if args.print_default_config:
            self.print_default_config()
            sys.exit()
        self.profile = args.profile
        self.application_name = args.application_name
        self.environment_name = args.environment_name
        self.region = args.region
        self.db_only = args.db_only
        self.no_db = args.no_db
        if self.db_only and self.no_db:
            raise Exception("--db-only cannot be used with --no-db")
        self.dir_path = os.path.dirname(os.path.realpath(__file__))
        self.config_file_path = args.config or os.path.join(self.dir_path, DEFAULT_CONFIG_FILE_PATH)
        # TODO: add support for application creation
        # self.create_new_application = False
        self.get_eb_config()

    def setup(self):
        # Read from config file
        config = self.parse_config_file()
        boto3.setup_default_session(profile_name=self.profile)
        # TODO: support worker tiers
        if not self.environment_name:
            self.environment_name = input("Input new environment name (lowercase-with-dashes): ")
        if self.db_only:
            cname_prefix = None
        else:
            cname_prefix = input("Input new CNAME prefix (lowercase-with-dashes): ")
        vpc_accessor = VPCAccessor(self.region)
        vpcs = vpc_accessor.get_vpcs()
        print("Current VPCS:")
        print(vpcs)
        if not(vpcs):
            raise Exception("No VPCs in that region")
        elif len(vpcs) == 1:
            vpc_id = list(vpcs.keys())[0]
            input(f"Using VPC {vpc_id}.  Press [Enter] to confirm ([Ctrl] + [C] to cancel)")
        else:
            vpc_id = input("Input vpc_id: ")
        eb_initializer = EBInitializer(self.region, config, self.application_name, self.environment_name, cname_prefix, vpc_id)
        if not self.db_only:
            print("\nLaunching EB environment")
            eb_initializer.set_up_environment()
            print("\nWaiting for EB environment to finish launching")
        application_security_group_id = eb_initializer.wait_for_environment()
        print("\nEB environment ready")
        
        if self.no_db:
            return
        
        # Call rds setup
        engine = Engine.postgres
        print("Setting up database")
        db_initializer = DatabaseInitializer(
            self.region, config, engine, vpc_id, self.environment_name, application_security_group_id
        )
        database_url = db_initializer.create_db()
        print("Database ready. Linking database to EB environment.")
        eb_initializer.update_environment_variables({
            "DATABASE_URL": database_url,
        })
        print("Environment setup complete.")
    
    def get_eb_config(self):
        """Parse eb config file if it exists. Otherwise, ask for user input and create file."""
        if os.path.isfile(EB_GLOBAL_CONFIG_FILE_PATH):
            with open(EB_GLOBAL_CONFIG_FILE_PATH) as config:
                global_configs = yaml.load(config, Loader=yaml.FullLoader)["global"]
            if not all(key in global_configs for key in ["application_name", "default_region", "profile"]):
                missing_keys = [
                    key for key in ["application_name", "default_region", "profile"] if key not
                    in global_configs
                ]
                raise Exception("MISSING GLOBAL CONFIG KEY(S): ", ", ".join(missing_keys))
            if not self.application_name:
                self.application_name = global_configs["application_name"]
            if not self.region:
                self.region = global_configs["default_region"]
            if not self.profile:
                self.profile = global_configs["profile"]
        
        current_profiles = boto3.session.Session().available_profiles
        if not self.profile:
            print("Current profiles")
            print(sorted(current_profiles))
            self.profile = input("Input profile: ")
        
        if self.profile not in current_profiles:
            raise Exception(f"Invalid profile {self.profile}")
        
        # Note that this must come after setting up profile so that we have the appropriate profile
        boto3.setup_default_session(profile_name=self.profile)
        ec2_client = boto3.client("ec2", "us-east-1")
        region_names = [i["RegionName"] for i in ec2_client.describe_regions()["Regions"]]
        if not self.region:
            print("AWS regions:")
            print(sorted(region_names))
            self.region = input("Input default region: ")
        if self.region not in region_names:
            raise Exception(f"Invalid region {self.region}")
        
        eb_client = boto3.client("elasticbeanstalk", self.region)
        current_applications = [i["ApplicationName"] for i in eb_client.describe_applications()["Applications"]]
        if not self.application_name:
            print("Current EB Applications:")
            print(sorted(current_applications))
            self.application_name = input("Input application name: ")
        if self.application_name not in current_applications:
            # TODO: create application if nonexistent
            # confirmation = input(f"Create new application named {self.application_name}? [y/n]")
            # if confirmation.lower().strip() in ["y", "yes"]:
            #     self.create_new_application = True
            # else:
            #     raise Exception(f"Invalid application name {self.application_name}")
            raise Exception(f"Invalid application name {self.application_name}")
        # Fill in config file if nonexistent
        if not os.path.isfile(EB_GLOBAL_CONFIG_FILE_PATH):
            self.create_eb_config_file()

        # Get existing environment for db-only calls
        if self.db_only:
            current_environments = [
                i["EnvironmentName"]
                for i in eb_client.describe_environments(ApplicationName=self.application_name)["Environments"]
            ]
            if not self.environment_name:
                print("Existing EB Environments:")
                print(sorted(current_environments))
                self.environment_name = input("Input environment name: ")
            if self.environment_name not in current_environments:
                confirmation = input(
                    f"Ignore --db-only flag and create new environment named {self.environment_name}? [y/n]"
                )
                if confirmation.lower().strip() in ["y", "yes"]:
                    self.db_only = False
                else:
                    raise Exception(f"Invalid environment name {self.environment_name}")
    
    def create_eb_config_file(self):
        config = yaml.dump(
            {
                "global": {
                    "application_name": self.application_name,
                    "default_region": self.region,
                    "profile": self.profile,
                }
            }
        )
        if not os.path.isdir(EB_GLOBAL_CONFIG_DIRECTORY):
            os.mkdir(EB_GLOBAL_CONFIG_DIRECTORY)
        with open(EB_GLOBAL_CONFIG_FILE_PATH, "w+") as config_file:
            config_file.write(config)

    def parse_config_file(self):
        with open(self.config_file_path) as config_file:
            configs = yaml.load(config_file, Loader=yaml.FullLoader)
        return configs

    def print_default_config(self):
        with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), DEFAULT_CONFIG_FILE_PATH)) as default_config_file:
            for line in default_config_file:
                print(line)


def main():
    SetupWrapper().setup()


if __name__ == "__main__":
    SetupWrapper().setup()



import boto3
import fnmatch
from choicesenum import ChoicesEnum
from eb_create_environment.vpc import VPCAccessor
from botocore.exceptions import ParamValidationError


class ServerTier(ChoicesEnum):
    web = "web"
    worker = "worker"


class EBInitializer(object):
    
    def __init__(self, region, config, application_name, environment_name, cname_prefix, vpc_id, server_tier=ServerTier.web):
        self.region = region
        self.config = config
        
        self.application_name = application_name
        self.environment_name = environment_name
        self.cname_prefix = cname_prefix
        self.vpc_id = vpc_id
        self.server_tier = server_tier
    
    def get_eb_client(self):
        return boto3.client("elasticbeanstalk", self.region)
    
    def get_config_param(self, param_name, subname=None):
        try:
            if subname:
                return self.config["ElasticBeanstalk"][param_name][subname]
            return self.config["ElasticBeanstalk"][param_name]
        except KeyError:
            return None
    
    def set_up_environment(self):
        if self.server_tier == ServerTier.web:
            tier_config = {
                "Name": "WebServer",
                "Type": "Standard",
            }
        elif self.server_tier == ServerTier.worker:
            tier_config = {
                "Name": "Worker",
                "Type": "SQS/HTTP",
            }
        else:
            raise Exception(f"invalid server tier: {self.server_tier}")
        
        vpc_accessor = VPCAccessor(self.region)
        instance_subnets = vpc_accessor.get_subnets(self.vpc_id, self.get_config_param("InstancePublicSubnets"), instance_type=self.get_config_param("InstanceTypes"))
        if not instance_subnets:
            raise Exception("No valid subnets for instances")
        print(f"Using the following subnets for instances: {instance_subnets}")
        
        if self.get_config_param("LoadBalancer"):
            load_balancer_subnets = vpc_accessor.get_subnets(self.vpc_id, self.get_config_param("LoadBalancer", "PublicSubnets"))
            if not load_balancer_subnets:
                raise Exception("No valid subnets for the load balancer")
            print(f"Using the following subnets for the load balancer: {load_balancer_subnets}")
        
        eb_client = self.get_eb_client()
        
        solution_stack_name = self.get_config_param("SolutionStackName")
        if "*" in solution_stack_name:
            resp = eb_client.list_available_solution_stacks()
            available_stacks = fnmatch.filter(resp["SolutionStacks"], solution_stack_name)
            if not available_stacks:
                raise Exception(f"No solution stacks match the provided pattern `{solution_stack_name}`")
            # Take the newest; according to Boto3 docks, solution stacks are listed with newest first
            solution_stack_name = available_stacks[0]
            print(f"Using solution stack: {solution_stack_name}")
        
        options = {
            ("aws:elasticbeanstalk:container:python", "NumProcesses"): str(self.get_config_param("NumProcesses")),
            ("aws:ec2:instances", "InstanceTypes"): self.get_config_param("InstanceTypes"),
            ("aws:autoscaling:launchconfiguration", "IamInstanceProfile"): self.get_config_param("IamInstanceProfile"),
            ("aws:elasticbeanstalk:environment:proxy", "ProxyServer"): self.get_config_param("ProxyServer"),
            ("aws:ec2:vpc", "VPCId"): self.vpc_id,
            ("aws:ec2:vpc", "ELBSubnets"): ",".join(load_balancer_subnets),
            ("aws:ec2:vpc", "Subnets"): ",".join(instance_subnets),
            ("aws:ec2:vpc", "AssociatePublicIpAddress"): "true" if self.get_config_param("AssociatePublicIpAddress") else "false",
        }
        
        if self.get_config_param("LoadBalancer"):
            options[("aws:elasticbeanstalk:environment", "EnvironmentType")] = "LoadBalanced"
            options[("aws:elasticbeanstalk:environment", "LoadBalancerType")] = str(self.get_config_param("LoadBalancer", "LoadBalancerType"))
            options[("aws:elb:loadbalancer", "LoadBalancerHTTPPort")] = "80"
            options[("aws:autoscaling:asg", "MinSize")] = str(self.get_config_param("LoadBalancer", "MinSize"))
            options[("aws:autoscaling:asg", "MaxSize")] = str(self.get_config_param("LoadBalancer", "MaxSize"))
            options[("aws:ec2:vpc", "ELBScheme")] = self.get_config_param("LoadBalancer", "ELBScheme")
            # Health check should get modified after initial deploy
            options[("aws:elb:healthcheck", "Target")] = "/"
        else:
            options[("aws:elasticbeanstalk:environment", "EnvironmentType")] = "SingleInstance"
        
        if self.get_config_param("ManagedUpdates"):
            options[("aws:elasticbeanstalk:managedactions", "ManagedActionsEnabled")] = "true"
            options[("aws:elasticbeanstalk:managedactions", "PreferredStartTime")] = self.get_config_param("ManagedUpdates", "PreferredStartTime")
            options[("aws:elasticbeanstalk:managedactions", "ServiceRoleForManagedUpdates")] = self.get_config_param("ManagedUpdates", "ServiceRoleForManagedUpdates")
            options[("aws:elasticbeanstalk:managedactions:platformupdate", "UpdateLevel")] = self.get_config_param("ManagedUpdates", "UpdateLevel")
        else:
            options[("aws:elasticbeanstalk:managedactions", "ManagedActionsEnabled")] = "false"
        
        if self.get_config_param("LoadBalancer") and self.get_config_param("LoadBalancer", "SSLCertificateId"):
            options[("aws:elb:loadbalancer", "LoadBalancerHTTPSPort")] = "443"
            options[("aws:elb:loadbalancer", "SSLCertificateId")] = self.get_config_param("LoadBalancer", "SSLCertificateId")
        
        # Note that we don't pass VersionLabel to intentionally deploy the sample app
        option_settings = [{"Namespace": key[0], "OptionName": key[1], "Value": value} for key, value in options.items()]
        try:
            eb_client.create_environment(
                ApplicationName=self.application_name,
                EnvironmentName=self.environment_name,
                CNAMEPrefix=self.cname_prefix,
                Tier=tier_config,
                SolutionStackName=solution_stack_name,
                OptionSettings=option_settings,
            )
        except ParamValidationError:
            for i, setting in enumerate(option_settings):
                print(i, setting)
            raise
    
    def wait_for_environment(self):
        eb_client = self.get_eb_client()
        waiter = eb_client.get_waiter('environment_exists')
        waiter.wait(
            ApplicationName=self.application_name,
            EnvironmentNames=[self.environment_name],
            IncludeDeleted=False,
            WaiterConfig={
                'Delay': 10,
                'MaxAttempts': 100
            }
        )
        env_resources = eb_client.describe_environment_resources(EnvironmentName=self.environment_name)
        launch_configuration_name = env_resources["EnvironmentResources"]["LaunchConfigurations"][0]["Name"]
        autoscaling_client = boto3.client("autoscaling", self.region)
        launch_configurations = autoscaling_client.describe_launch_configurations(LaunchConfigurationNames=[launch_configuration_name])
        return launch_configurations["LaunchConfigurations"][0]["SecurityGroups"][0]

    def update_environment_variables(self, environment_variable_mapping):
        option_settings = [
            {
                "Namespace": "aws:elasticbeanstalk:application:environment",
                "OptionName": variable,
                "Value": value,
            }
            for variable, value in environment_variable_mapping.items()
        ]
        return self.get_eb_client().update_environment(
            ApplicationName=self.application_name,
            EnvironmentName=self.environment_name,
            OptionSettings=option_settings
        )
